@@ -22,12 +22,13 @@ pub fn fls() -> Term { abs(abs(Var(1))) }
 /// # #[macro_use] extern crate lambda_calculus;
 /// # fn main() {
 /// use lambda_calculus::booleans::{and, tru, fls};
-/// use lambda_calculus::reduction::beta_full;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta_full(app!(and(), tru(), tru())), tru());
-/// assert_eq!(beta_full(app!(and(), tru(), fls())), fls());
-/// assert_eq!(beta_full(app!(and(), fls(), tru())), fls());
-/// assert_eq!(beta_full(app!(and(), fls(), fls())), fls());
+/// assert_eq!(beta(app!(and(), tru(), tru()), &Normal, 0), tru());
+/// assert_eq!(beta(app!(and(), tru(), fls()), &Normal, 0), fls());
+/// assert_eq!(beta(app!(and(), fls(), tru()), &Normal, 0), fls());
+/// assert_eq!(beta(app!(and(), fls(), fls()), &Normal, 0), fls());
 /// # }
 /// ```
 pub fn and() -> Term {
@@ -45,12 +46,13 @@ pub fn and() -> Term {
 /// # #[macro_use] extern crate lambda_calculus;
 /// # fn main() {
 /// use lambda_calculus::booleans::{or, tru, fls};
-/// use lambda_calculus::reduction::beta_full;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta_full(app!(or(), tru(), tru())), tru());
-/// assert_eq!(beta_full(app!(or(), tru(), fls())), tru());
-/// assert_eq!(beta_full(app!(or(), fls(), tru())), tru());
-/// assert_eq!(beta_full(app!(or(), fls(), fls())), fls());
+/// assert_eq!(beta(app!(or(), tru(), tru()), &Normal, 0), tru());
+/// assert_eq!(beta(app!(or(), tru(), fls()), &Normal, 0), tru());
+/// assert_eq!(beta(app!(or(), fls(), tru()), &Normal, 0), tru());
+/// assert_eq!(beta(app!(or(), fls(), fls()), &Normal, 0), fls());
 /// # }
 /// ```
 pub fn or() -> Term {
@@ -68,10 +70,11 @@ pub fn or() -> Term {
 /// # #[macro_use] extern crate lambda_calculus;
 /// # fn main() {
 /// use lambda_calculus::booleans::{not, tru, fls};
-/// use lambda_calculus::reduction::beta_full;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta_full(app!(not(), tru())), fls());
-/// assert_eq!(beta_full(app!(not(), fls())), tru());
+/// assert_eq!(beta(app!(not(), tru()), &Normal, 0), fls());
+/// assert_eq!(beta(app!(not(), fls()), &Normal, 0), tru());
 /// # }
 /// ```
 pub fn not() -> Term {
@@ -89,17 +92,18 @@ pub fn not() -> Term {
 /// # #[macro_use] extern crate lambda_calculus;
 /// # fn main() {
 /// use lambda_calculus::booleans::{xor, tru, fls};
-/// use lambda_calculus::reduction::beta_full;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta_full(app!(xor(), tru(), tru())), fls());
-/// assert_eq!(beta_full(app!(xor(), tru(), fls())), tru());
-/// assert_eq!(beta_full(app!(xor(), fls(), tru())), tru());
-/// assert_eq!(beta_full(app!(xor(), fls(), fls())), fls());
+/// assert_eq!(beta(app!(xor(), tru(), tru()), &Normal, 0), fls());
+/// assert_eq!(beta(app!(xor(), tru(), fls()), &Normal, 0), tru());
+/// assert_eq!(beta(app!(xor(), fls(), tru()), &Normal, 0), tru());
+/// assert_eq!(beta(app!(xor(), fls(), fls()), &Normal, 0), fls());
 /// # }
 /// ```
 pub fn xor() -> Term {
     abs(abs(
-        app!(Var(2), app!(not(), Var(1)), Var(1))
+        app!(Var(2), app(not(), Var(1)), Var(1))
     ))
 }
 
@@ -114,10 +118,11 @@ pub fn xor() -> Term {
 /// # fn main() {
 /// use lambda_calculus::booleans::{if_else, tru, fls};
 /// use lambda_calculus::arithmetic::{zero, one};
-/// use lambda_calculus::reduction::beta_full;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta_full(app!(if_else(), tru(), one(), zero())), one());
-/// assert_eq!(beta_full(app!(if_else(), fls(), one(), zero())), zero());
+/// assert_eq!(beta(app!(if_else(), tru(), one(), zero()), &Normal, 0), one());
+/// assert_eq!(beta(app!(if_else(), fls(), one(), zero()), &Normal, 0), zero());
 /// # }
 /// ```
 pub fn if_else() -> Term {
