@@ -7,12 +7,11 @@
 // //! * the recursion combinator U - needs more research
 //! * the looping combinator ω
 //! * the divergent combinator Ω
-//! * [the fixed-point combinator Y](https://en.wikipedia.org/wiki/Fixed-point_combinator)
+//! * [the fixed-point combinators Y and Z](https://en.wikipedia.org/wiki/Fixed-point_combinator)
 //! * the thrush (reverse application) combinator T
 
 use term::*;
 use term::Term::*;
-use reduction::{EVALUATION_ORDER, Order};
 
 /// I - the identity combinator.
 ///
@@ -22,9 +21,10 @@ use reduction::{EVALUATION_ORDER, Order};
 /// ```
 /// use lambda_calculus::combinators::i;
 /// use lambda_calculus::arithmetic::zero;
-/// use lambda_calculus::reduction::beta_full;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta_full(i().app(zero())), zero());
+/// assert_eq!(beta(i().app(zero()), &Normal, 0), zero());
 /// ```
 pub fn i() -> Term { abs(Var(1)) }
 
@@ -38,9 +38,10 @@ pub fn i() -> Term { abs(Var(1)) }
 /// # fn main() {
 /// use lambda_calculus::combinators::k;
 /// use lambda_calculus::arithmetic::{zero, one};
-/// use lambda_calculus::reduction::beta_full;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta_full(app!(k(), zero(), one())), zero());
+/// assert_eq!(beta(app!(k(), zero(), one()), &Normal, 0), zero());
 /// # }
 /// ```
 pub fn k() -> Term { abs(abs(Var(2))) }
@@ -55,17 +56,17 @@ pub fn k() -> Term { abs(abs(Var(2))) }
 /// # fn main() {
 /// use lambda_calculus::term::Term;
 /// use lambda_calculus::combinators::s;
-/// use lambda_calculus::reduction::beta_full;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta_full(app!(s(), 0.into(), 1.into(), 2.into())),
-///            beta_full(app!(Term::from(0), 2.into(), app!(Term::from(1), 2.into()))));
+/// assert_eq!(beta(app!(s(), 0.into(), 1.into(), 2.into()), &Normal, 0),
+///            beta(app!(Term::from(0), 2.into(), app!(Term::from(1), 2.into())), &Normal, 0)
+/// );
 /// # }
 /// ```
 pub fn s() -> Term {
     abs(abs(abs(
-        Var(3)
-        .app(Var(1))
-        .app(Var(2).app(Var(1)))
+        app!(Var(3), Var(1), app(Var(2), Var(1)))
     )))
 }
 
@@ -78,11 +79,13 @@ pub fn s() -> Term {
 /// # #[macro_use] extern crate lambda_calculus;
 /// # fn main() {
 /// use lambda_calculus::combinators::{iota, i, k, s};
-/// use lambda_calculus::reduction::beta_full;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta_full(app!(iota(), iota())), i());
-/// assert_eq!(beta_full(app!(iota(), app!(iota(), app!(iota(), iota())))), k());
-/// assert_eq!(beta_full(app!(iota(), app!(iota(), app!(iota(), app!(iota(), iota()))))), s());
+/// assert_eq!(beta(app!(iota(), iota()), &Normal, 0), i());
+/// assert_eq!(beta(app!(iota(), app!(iota(), app!(iota(), iota()))), &Normal, 0), k());
+/// assert_eq!(beta(app!(iota(), app!(iota(), app!(iota(), app!(iota(), iota())))), &Normal, 0),
+///            s());
 /// # }
 /// ```
 pub fn iota() -> Term { abs(app!(Var(1), s(), k())) }
@@ -97,15 +100,17 @@ pub fn iota() -> Term { abs(app!(Var(1), s(), k())) }
 /// # fn main() {
 /// use lambda_calculus::term::Term;
 /// use lambda_calculus::combinators::b;
-/// use lambda_calculus::reduction::beta_full;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta_full(app!(b(), 0.into(), 1.into(), 2.into())),
-///            beta_full(app!(Term::from(0), app!(Term::from(1), 2.into()))));
+/// assert_eq!(beta(app!(b(), 0.into(),           1.into(), 2.into() ), &Normal, 0),
+///            beta(app!(Term::from(0), app!(Term::from(1), 2.into())), &Normal, 0)
+/// );
 /// # }
 /// ```
 pub fn b() -> Term {
     abs(abs(abs(
-        app!(Var(3), app!(Var(2), Var(1)))
+        app(Var(3), app(Var(2), Var(1)))
     )))
 }
 
@@ -119,10 +124,12 @@ pub fn b() -> Term {
 /// # fn main() {
 /// use lambda_calculus::term::Term;
 /// use lambda_calculus::combinators::c;
-/// use lambda_calculus::reduction::beta_full;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta_full(app!(c(), 0.into(), 1.into(), 2.into())),
-///            beta_full(app!(Term::from(0), 2.into(), 1.into())));
+/// assert_eq!(beta(app!(c(), 0.into(), 1.into(), 2.into()), &Normal, 0),
+///            beta(app!(Term::from(0), 2.into(), 1.into()), &Normal, 0)
+/// );
 /// # }
 /// ```
 pub fn c() -> Term {
@@ -141,10 +148,12 @@ pub fn c() -> Term {
 /// # fn main() {
 /// use lambda_calculus::combinators::w;
 /// use lambda_calculus::arithmetic::{zero, one};
-/// use lambda_calculus::reduction::beta_full;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta_full(app!(w(), zero(), one())),
-///            beta_full(app!(zero(), one(), one())));
+/// assert_eq!(beta(app!(   w(), zero(), one()), &Normal, 0),
+///            beta(app!(zero(),  one(), one()), &Normal, 0)
+/// );
 /// # }
 /// ```
 pub fn w() -> Term {
@@ -166,9 +175,12 @@ pub fn u() -> Term { abs(abs(Var(1).app(Var(2).app(Var(2)).app(Var(1))))) }
 /// ```
 /// use lambda_calculus::combinators::om;
 /// use lambda_calculus::arithmetic::zero;
-/// use lambda_calculus::reduction::beta_full;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta_full(om().app(zero())), beta_full(zero().app(zero())));
+/// assert_eq!(beta(  om().app(zero()), &Normal, 0),
+///            beta(zero().app(zero()), &Normal, 0)
+/// );
 /// ```
 pub fn om() -> Term { abs(Var(1).app(Var(1))) }
 
@@ -179,13 +191,15 @@ pub fn om() -> Term { abs(Var(1).app(Var(1))) }
 /// # Example
 /// ```
 /// use lambda_calculus::combinators::omm;
-/// use lambda_calculus::reduction::beta_once;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta_once(omm()), omm());
+/// assert_eq!(beta(omm(), &Normal, 3), omm());
 /// ```
 pub fn omm() -> Term { om().app(om()) }
 
-/// Y - the fixed-point combinator.
+/// Y - the lazy fixed-point combinator suitable for `Normal` and `HybridNormal` β-reduction
+/// `Order`s.
 ///
 /// Y := λf.(λx.f (x x)) (λx.f (x x)) = λ (λ 2 (1 1)) (λ 2 (1 1))
 ///
@@ -195,28 +209,45 @@ pub fn omm() -> Term { om().app(om()) }
 /// # fn main() {
 /// use lambda_calculus::combinators::y;
 /// use lambda_calculus::arithmetic::zero;
-/// use lambda_calculus::reduction::beta_full;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta_full(y().app(zero())), beta_full(app!(zero(), app!(y(), zero()))));
+/// assert_eq!(beta(             app!(y(), zero() ), &Normal, 0),
+///            beta(app!(zero(), app!(y(), zero())), &Normal, 0)
+/// );
 /// # }
 /// ```
 pub fn y() -> Term {
-    match EVALUATION_ORDER {
-        Order::Normal => {
-            abs(app(
-                abs(app!(Var(2), app!(Var(1), Var(1)))),
-                abs(app!(Var(2), app!(Var(1), Var(1))))
-            ))
-        }
-        Order::ApplicativeLeft | Order::ApplicativeRight => { /* should any variant work? */
-            panic!("Y combinator works only with normal evaluation order")
-            //parse(&"(λλ212)(λλ2(121))").unwrap()
-            //parse(&"(λλ(1(λ3321)))(λλ(1(λ3321)))").unwrap()
-            //parse(&"(λ11)(λ(λ(3(22)1)))").unwrap()
-            //parse(&"λ(λ(2(λ(221))))(λ(2(λ(221))))").unwrap()
-            //parse(&"λ(λ11)(λ(2(λ((22)1))))").unwrap()
-        }
-    }
+    abs(app(
+        abs(app(Var(2), app(Var(1), Var(1)))),
+        abs(app(Var(2), app(Var(1), Var(1))))
+    ))
+}
+
+/// Z - the strict fixed-point combinator suitable for `Normal`, `HybridNormal`, `CallByValue` and
+/// `HybridApplicative` β-reduction `Order`s.
+///
+/// Z := λf.(λx.f (λv.x x v)) (λx.f (λv.x x v)) = λ (λ 2 (λ 2 2 1)) (λ 2 (λ 2 2 1))
+///
+/// # Example
+/// ```
+/// # #[macro_use] extern crate lambda_calculus;
+/// # fn main() {
+/// use lambda_calculus::combinators::z;
+/// use lambda_calculus::arithmetic::zero;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
+///
+/// assert_eq!(beta(             app!(z(), zero() ), &CallByValue, 0),
+///            beta(app!(zero(), app!(z(), zero())), &CallByValue, 0)
+/// );
+/// # }
+/// ```
+pub fn z() -> Term {
+    abs(app(
+        abs(app(Var(2), abs(app!(Var(2), Var(2), Var(1))))),
+        abs(app(Var(2), abs(app!(Var(2), Var(2), Var(1)))))
+    ))
 }
 
 /// T - the thrush combinator
