@@ -6,11 +6,11 @@ use term::Error::*;
 use booleans::*;
 use pair::*;
 use arithmetic::{zero, succ};
-use combinators::y;
+use combinators::{y, i};
 use std::ops::Index;
 
 /// Equivalent to `booleans::fls()`; produces a Church-encoded `nil`, the last link of a
-/// Church-encoded list.
+/// Church list.
 ///
 /// NIL := FALSE
 ///
@@ -23,7 +23,7 @@ use std::ops::Index;
 /// ```
 pub fn nil() -> Term { fls() }
 
-/// Applied to a Church-encoded list it determines if it is empty.
+/// Applied to a Church list it determines if it is empty.
 ///
 /// NULL := λl.l (λhtd.FALSE) TRUE = λ 1 (λ λ λ FALSE) TRUE
 ///
@@ -33,16 +33,17 @@ pub fn nil() -> Term { fls() }
 /// # fn main() {
 /// use lambda_calculus::list::{nil, null};
 /// use lambda_calculus::booleans::tru;
-/// use lambda_calculus::reduction::beta_full;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta_full(app!(null(), nil())), tru());
+/// assert_eq!(beta(app!(null(), nil()), &Normal, 0), tru());
 /// # }
 /// ```
 pub fn null() -> Term {
     abs(app!(Var(1), abs(abs(abs(fls()))), tru()))
 }
 
-/// Equivalent to `pair::pair()`; applied to two terms it returns them contained in a Church-encoded list.
+/// Equivalent to `pair::pair()`; applied to two terms it returns them contained in a Church list.
 ///
 /// CONS := PAIR
 ///
@@ -53,9 +54,10 @@ pub fn null() -> Term {
 /// use lambda_calculus::term::Term;
 /// use lambda_calculus::arithmetic::{zero, one};
 /// use lambda_calculus::list::{nil, cons};
-/// use lambda_calculus::reduction::beta_full;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
 ///
-/// let list_110_consed = beta_full(
+/// let list_110_consed = beta(
 ///     app!(
 ///         cons(),
 ///         one(),
@@ -68,8 +70,7 @@ pub fn null() -> Term {
 ///                 nil()
 ///             )
 ///         )
-///     )
-///
+///     ), &Normal, 0
 /// );
 /// let list_110_from_vec = Term::from(vec![one(), one(), zero()]);
 ///
@@ -78,7 +79,7 @@ pub fn null() -> Term {
 /// ```
 pub fn cons() -> Term { pair() }
 
-/// Equivalent to `pair::first()`; applied to a Church-encoded list it returns its first element.
+/// Equivalent to `pair::first()`; applied to a Church list it returns its first element.
 ///
 /// HEAD := FIRST
 ///
@@ -89,16 +90,17 @@ pub fn cons() -> Term { pair() }
 /// use lambda_calculus::term::Term;
 /// use lambda_calculus::list::head;
 /// use lambda_calculus::arithmetic::{zero, one};
-/// use lambda_calculus::reduction::beta_full;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
 ///
 /// let list_110 = Term::from(vec![one(), one(), zero()]);
 ///
-/// assert_eq!(beta_full(app!(head(), list_110)), one());
+/// assert_eq!(beta(app!(head(), list_110), &Normal, 0), one());
 /// # }
 /// ```
 pub fn head() -> Term { first() }
 
-/// Equivalent to `pair::second()`; applied to a Church-encoded list it returns a new list with all its
+/// Equivalent to `pair::second()`; applied to a Church list it returns a new list with all its
 /// elements but the first one.
 ///
 /// TAIL := SECOND
@@ -110,16 +112,17 @@ pub fn head() -> Term { first() }
 /// use lambda_calculus::term::Term;
 /// use lambda_calculus::list::tail;
 /// use lambda_calculus::arithmetic::{zero, one};
-/// use lambda_calculus::reduction::beta_full;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
 ///
 /// let list_110 = Term::from(vec![one(), one(), zero()]);
 ///
-/// assert_eq!(beta_full(app!(tail(), list_110)), Term::from(vec![one(), zero()]));
+/// assert_eq!(beta(app!(tail(), list_110), &Normal, 0), Term::from(vec![one(), zero()]));
 /// # }
 /// ```
 pub fn tail() -> Term { second() }
 
-/// Applied to a Church-encoded list it returns its Church-encoded length.
+/// Applied to a Church list it returns its Church-encoded length.
 ///
 /// LENGTH := Y (λgcx.NULL x c (g (SUCC c) (SECOND x))) ZERO
 /// = Y (λλλ NULL 1 2 (3 (SUCC 2) (SECOND 1))) ZERO
@@ -130,12 +133,13 @@ pub fn tail() -> Term { second() }
 /// # fn main() {
 /// use lambda_calculus::term::Term;
 /// use lambda_calculus::list::{length, nil};
-/// use lambda_calculus::reduction::beta_full;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
 ///
 /// let list_4 = Term::from(vec![1.into(), 1.into(), 0.into(), 1.into()]);
 ///
-/// assert_eq!(beta_full(app!(length(), nil())),  0.into());
-/// assert_eq!(beta_full(app!(length(), list_4)), 4.into());
+/// assert_eq!(beta(app!(length(), nil() ), &Normal, 0), 0.into());
+/// assert_eq!(beta(app!(length(), list_4), &Normal, 0), 4.into());
 /// # }
 /// ```
 pub fn length() -> Term {
@@ -148,8 +152,8 @@ pub fn length() -> Term {
                 Var(2),
                 app!(
                     Var(3),
-                    app!(succ(), Var(2)),
-                    app!(second(), Var(1))
+                    app(succ(), Var(2)),
+                    app(second(), Var(1))
                 )
             )
         ))),
@@ -157,7 +161,7 @@ pub fn length() -> Term {
     )
 }
 
-/// Reverses a Church-encoded list.
+/// Reverses a Church list.
 ///
 /// REVERSE := Y (λgal.NULL l a (g (PAIR (FIRST l) a) (SECOND l))) NIL =
 /// Y (λ λ λ NULL 1 2 (3 (PAIR (FIRST 1) 2) (SECOND 1))) NIL
@@ -169,11 +173,12 @@ pub fn length() -> Term {
 /// use lambda_calculus::term::Term;
 /// use lambda_calculus::list::reverse;
 /// use lambda_calculus::arithmetic::{zero, one};
-/// use lambda_calculus::reduction::beta_full;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
 ///
 /// let list = Term::from(vec![one(), one(), zero()]);
 ///
-/// assert_eq!(beta_full(app!(reverse(), list)), Term::from(vec![zero(), one(), one()]));
+/// assert_eq!(beta(app!(reverse(), list), &Normal, 0), Term::from(vec![zero(), one(), one()]));
 /// # }
 /// ```
 pub fn reverse() -> Term {
@@ -186,8 +191,8 @@ pub fn reverse() -> Term {
                 Var(2),
                 app!(
                     Var(3),
-                    app!(pair(), app!(first(), Var(1)), Var(2)),
-                    app!(second(), Var(1))
+                    app!(pair(), app(first(), Var(1)), Var(2)),
+                    app(second(), Var(1))
                 )
             )
         ))),
@@ -195,7 +200,7 @@ pub fn reverse() -> Term {
     )
 }
 
-/// Applied to a Church-encoded number `n` and `n` `Term`s it creates a Church-encoded list of
+/// Applied to a Church number `n` and `n` `Term`s it creates a Church list of
 /// those terms.
 ///
 /// LIST := λn.n (λfax.f (PAIR x a)) REVERSE NIL = λ 1 (λ λ λ 3 (PAIR 1 2)) REVERSE NIL
@@ -206,9 +211,10 @@ pub fn reverse() -> Term {
 /// # fn main() {
 /// use lambda_calculus::term::Term;
 /// use lambda_calculus::list::list;
-/// use lambda_calculus::reduction::beta_full;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta_full(app!(list(), 3.into(), 0.into(), 1.into(), 1.into())),
+/// assert_eq!(beta(app!(list(), 3.into(), 0.into(), 1.into(), 1.into()), &Normal, 0),
 ///            Term::from(vec![0.into(), 1.into(), 1.into()]));
 /// # }
 /// ```
@@ -217,7 +223,7 @@ pub fn list() -> Term {
         app!(
             Var(1),
             abs(abs(abs(
-                app!(Var(3), app!(pair(), Var(1), Var(2)))
+                app(Var(3), app!(pair(), Var(1), Var(2)))
             ))),
             reverse(),
             nil()
@@ -225,7 +231,7 @@ pub fn list() -> Term {
     )
 }
 
-/// Applied to 2 Church-encoded lists it concatenates them.
+/// Applied to 2 Church lists it concatenates them.
 ///
 /// APPEND := Y (λgab. NULL a b (PAIR (FIRST a) (g (SECOND a) b))) =
 /// Y (λ λ λ NULL 2 1 (PAIR (FIRST 2) (3 (SECOND 2) 1)))
@@ -236,18 +242,18 @@ pub fn list() -> Term {
 /// # fn main() {
 /// use lambda_calculus::term::Term;
 /// use lambda_calculus::list::append;
-/// use lambda_calculus::reduction::beta_full;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
 ///
 /// let list1 = Term::from(vec![0.into(), 1.into()]);
 /// let list2 = Term::from(vec![2.into(), 3.into()]);
 ///
-/// assert_eq!(beta_full(app!(append(), list1, list2)),
+/// assert_eq!(beta(app!(append(), list1, list2), &Normal, 0),
 ///            Term::from(vec![0.into(), 1.into(), 2.into(), 3.into()]));
 /// # }
 /// ```
 pub fn append() -> Term {
-    app!(
-        y(),
+    y().app(
         abs(abs(abs(
             app!(
                 null(),
@@ -255,15 +261,15 @@ pub fn append() -> Term {
                 Var(1),
                 app!(
                     pair(),
-                    app!(first(), Var(2)),
-                    app!(Var(3), app!(second(), Var(2)), Var(1))
+                    app(first(), Var(2)),
+                    app!(Var(3), app(second(), Var(2)), Var(1))
                 )
             )
         )))
     )
 }
 
-/// Applied to a Church-encoded number `i` and a Church-encoded list it returns the `i`-th
+/// Applied to a Church number `i` and a Church list it returns the `i`-th
 /// (zero-indexed) element of the list.
 ///
 /// INDEX := λix. FIRST (x SECOND i) = λ λ FIRST (2 SECOND 1)
@@ -274,22 +280,190 @@ pub fn append() -> Term {
 /// # fn main() {
 /// use lambda_calculus::term::Term;
 /// use lambda_calculus::list::index;
-/// use lambda_calculus::reduction::beta_full;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
 ///
 /// let list = Term::from(vec![3.into(), 4.into(), 5.into()]);
 ///
-/// assert_eq!(beta_full(app!(index(), 0.into(), list.clone())), 3.into());
-/// assert_eq!(beta_full(app!(index(), 2.into(), list)        ), 5.into());
+/// assert_eq!(beta(app!(index(), 0.into(), list.clone()), &Normal, 0), 3.into());
+/// assert_eq!(beta(app!(index(), 2.into(), list        ), &Normal, 0), 5.into());
 /// # }
 /// ```
 pub fn index() -> Term {
     abs(abs(
-        app!(first(), app!(Var(2), second(), Var(1)))
+        app(first(), app!(Var(2), second(), Var(1)))
     ))
 }
 
+/// Applied to a function and a Church list it maps the function over it.
+///
+/// MAP := Y (λgfx. NULL x NIL (PAIR (f (FIRST x)) (g f (SECOND x)))) =
+/// Y (λ λ λ NULL 1 NIL (PAIR (2 (FIRST 1)) (3 2 (SECOND 1))))
+///
+/// # Example
+/// ```
+/// # #[macro_use] extern crate lambda_calculus;
+/// # fn main() {
+/// use lambda_calculus::term::Term;
+/// use lambda_calculus::list::map;
+/// use lambda_calculus::arithmetic::succ;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
+///
+/// let list = Term::from(vec![1.into(), 2.into(), 3.into()]);
+///
+/// assert_eq!(beta(app!(map(), succ(), list), &Normal, 0),
+///            Term::from(vec![2.into(), 3.into(), 4.into()]));
+/// # }
+/// ```
+pub fn map() -> Term {
+    y().app(
+        abs(abs(abs(
+            app!(
+                null(),
+                Var(1),
+                nil(),
+                app!(
+                    pair(),
+                    app(Var(2), app(first(), Var(1))),
+                    app!(Var(3), Var(2), app(second(), Var(1)))
+                )
+            )
+        )))
+    )
+}
+
+/// Applied to a function, a starting value and a Church list it performs a
+/// [left fold](https://en.wikipedia.org/wiki/Fold_(higher-order_function)#Folds_on_lists) on the
+/// list.
+///
+/// FOLDL := Y (λgfex. NULL x e (g f (f e (FIRST x)) (SECOND x))) =
+/// Y (λ λ λ λ NULL 1 2 (4 3 (3 2 (FIRST 1)) (SECOND 1)))
+///
+/// # Example
+/// ```
+/// # #[macro_use] extern crate lambda_calculus;
+/// # fn main() {
+/// use lambda_calculus::term::Term;
+/// use lambda_calculus::list::{foldl, nil};
+/// use lambda_calculus::arithmetic::plus;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
+///
+/// let list = Term::from(vec![1.into(), 2.into(), 3.into()]);
+///
+/// assert_eq!(beta(app!(foldl(), plus(), 0.into(), list ), &Normal, 0), 6.into());
+/// assert_eq!(beta(app!(foldl(), plus(), 0.into(), nil()), &Normal, 0), 0.into());
+/// # }
+/// ```
+pub fn foldl() -> Term {
+    y().app(
+        abs(abs(abs(abs(
+            app!(
+                null(),
+                Var(1),
+                Var(2),
+                app!(
+                    Var(4),
+                    Var(3),
+                    app!(Var(3), Var(2), app(first(), Var(1))),
+                    app(second(), Var(1))
+                )
+            )
+        ))))
+    )
+}
+
+/// Applied to a function, a starting value and a Church list it performs a
+/// [right fold](https://en.wikipedia.org/wiki/Fold_(higher-order_function)#Folds_on_lists) on the
+/// list.
+///
+/// FOLDR := λfex. Y (λgy. NULL y e (f (FIRST y) (g (SECOND y)))) x =
+/// λ λ λ Y (λ λ NULL 1 4 (5 (FIRST 1) (2 (SECOND 1)))) 1
+///
+/// # Example
+/// ```
+/// # #[macro_use] extern crate lambda_calculus;
+/// # fn main() {
+/// use lambda_calculus::term::Term;
+/// use lambda_calculus::list::{foldr, nil};
+/// use lambda_calculus::arithmetic::plus;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
+///
+/// let list = Term::from(vec![1.into(), 2.into(), 3.into()]);
+///
+/// assert_eq!(beta(app!(foldr(), plus(), 0.into(), list ), &Normal, 0), 6.into());
+/// assert_eq!(beta(app!(foldr(), plus(), 0.into(), nil()), &Normal, 0), 0.into());
+/// # }
+/// ```
+pub fn foldr() -> Term {
+    abs(abs(abs(
+        app!(
+            y(),
+            abs(abs(
+                app!(
+                    null(),
+                    Var(1),
+                    Var(4),
+                    app!(
+                        Var(5),
+                        app(first(), Var(1)),
+                        app(Var(2), app(second(), Var(1)))
+                    )
+                )
+            )),
+            Var(1)
+        )
+    )))
+}
+
+/// Applied to a predicate and a Church list it filters the list based on the predicate.
+///
+/// FILTER := Y (λgfx. NULL x NIL (f (FIRST x) (PAIR (FIRST x)) I (g f (SECOND x)))) =
+/// Y (λ λ λ NULL 1 NIL (2 (FIRST 1) (PAIR (FIRST 1)) I (3 2 (SECOND 1))))
+///
+/// # Example
+/// ```
+/// # #[macro_use] extern crate lambda_calculus;
+/// # fn main() {
+/// use lambda_calculus::term::Term;
+/// use lambda_calculus::list::{filter, nil};
+/// use lambda_calculus::arithmetic::{is_zero, gt};
+/// use lambda_calculus::combinators::c;
+/// use lambda_calculus::reduction::beta;
+/// use lambda_calculus::reduction::Order::*;
+///
+/// let list = Term::from(vec![0.into(), 1.into(), 2.into(), 3.into()]);
+/// let gt1  = app!(c(), gt(), 1.into());
+///
+/// assert_eq!(beta(app!(filter(), is_zero(), list.clone()), &Normal, 0),
+///            Term::from(vec![0.into()]));
+/// assert_eq!(beta(app!(filter(), gt1, list), &Normal, 0),
+///            Term::from(vec![2.into(), 3.into()]));
+/// # }
+/// ```
+pub fn filter() -> Term {
+    y().app(
+        abs(abs(abs(
+            app!(
+                null(),
+                Var(1),
+                nil(),
+                app!(
+                    Var(2),
+                    app(first(), Var(1)),
+                    app(pair(), app(first(), Var(1))),
+                    i(),
+                    app!(Var(3), Var(2), app(second(), Var(1)))
+                )
+            )
+        )))
+    )
+}
+
 impl Term {
-    /// Checks whether self is a Church-encoded empty list, i.e. `nil()`.
+    /// Checks whether self is a empty Church list, i.e. `nil()`.
     ///
     /// # Example
     /// ```
@@ -301,7 +475,7 @@ impl Term {
         *self == nil()
     }
 
-    // Returns a reference to the last term of a Church-encoded list.
+    // Returns a reference to the last term of a Church list.
     fn last_ref(&self) -> Result<&Term, Error> {
         if !self.is_pair() { return Err(NotAList) }
 
@@ -314,7 +488,7 @@ impl Term {
         Ok(last_candidate)
     }
 
-    /// Checks whether `self` is a Church-encoded list.
+    /// Checks whether `self` is a Church list.
     ///
     /// # Example
     /// ```
@@ -329,7 +503,7 @@ impl Term {
         self.last_ref() == Ok(&nil())
     }
 
-    /// Splits a Church-encoded list into a pair containing its first term and a list of all the
+    /// Splits a Church list into a pair containing its first term and a list of all the
     /// other terms, consuming `self`.
     ///
     /// # Example
@@ -349,7 +523,7 @@ impl Term {
         }
     }
 
-    /// Splits a Church-encoded list into a pair containing references to its first term and a to
+    /// Splits a Church list into a pair containing references to its first term and a to
     /// list of all the other terms.
     ///
     /// # Example
@@ -369,7 +543,7 @@ impl Term {
         }
     }
 
-    /// Splits a Church-encoded list into a pair containing mutable references to its first term
+    /// Splits a Church list into a pair containing mutable references to its first term
     /// and a to list of all the other terms.
     ///
     /// # Example
@@ -390,7 +564,7 @@ impl Term {
         }
     }
 
-    /// Returns the first term from a Church-encoded list, consuming `self`.
+    /// Returns the first term from a Church list, consuming `self`.
     ///
     /// # Example
     /// ```
@@ -405,7 +579,7 @@ impl Term {
         Ok(try!(self.uncons()).0)
     }
 
-    /// Returns a reference to the first term of a Church-encoded list.
+    /// Returns a reference to the first term of a Church list.
     ///
     /// # Example
     /// ```
@@ -420,7 +594,7 @@ impl Term {
         Ok(try!(self.uncons_ref()).0)
     }
 
-    /// Returns a mutable reference to the first term of a Church-encoded list.
+    /// Returns a mutable reference to the first term of a Church list.
     ///
     /// # Example
     /// ```
@@ -435,7 +609,7 @@ impl Term {
         Ok(try!(self.uncons_ref_mut()).0)
     }
 
-    /// Returns a list of all the terms of a Church-encoded list but the first one, consuming `self`.
+    /// Returns a list of all the terms of a Church list but the first one, consuming `self`.
     ///
     /// # Example
     /// ```
@@ -450,7 +624,7 @@ impl Term {
         Ok(try!(self.uncons()).1)
     }
 
-    /// Returns a reference to a list of all the terms of a Church-encoded list but the first one.
+    /// Returns a reference to a list of all the terms of a Church list but the first one.
     ///
     /// # Example
     /// ```
@@ -465,7 +639,7 @@ impl Term {
         Ok(try!(self.uncons_ref()).1)
     }
 
-    /// Returns a mutable reference to a list of all the terms of a Church-encoded list but the
+    /// Returns a mutable reference to a list of all the terms of a Church list but the
     /// first one.
     ///
     /// # Example
@@ -481,7 +655,7 @@ impl Term {
         Ok(try!(self.uncons_ref_mut()).1)
     }
 
-    /// Returns the length of a Church-encoded list
+    /// Returns the length of a Church list
     ///
     /// # Example
     /// ```
@@ -504,7 +678,7 @@ impl Term {
         Ok(n)
     }
 
-    /// Adds a term to the beginning of a Church-encoded list and returns the new list. Consumes
+    /// Adds a term to the beginning of a Church list and returns the new list. Consumes
     /// `self` and the argument.
     ///
     /// # Example
@@ -520,7 +694,7 @@ impl Term {
         abs(Var(1).app(term).app(self))
     }
 
-    /// Removes the first element from a Church-encoded list and returns it.
+    /// Removes the first element from a Church list and returns it.
     ///
     /// # Example
     /// ```
@@ -585,7 +759,8 @@ impl Index<usize> for Term {
 #[cfg(test)]
 mod test {
     use super::*;
-    use reduction::beta_full;
+    use reduction::beta;
+    use reduction::Order::*;
 
     #[test]
     fn empty_list() {
@@ -596,7 +771,7 @@ mod test {
     #[test]
     fn list_push() {
         let list_pushed = nil().push(0.into()).push(1.into()).push(1.into());
-        let list_consed = beta_full(
+        let list_consed = beta(
             app!(
                 cons(),
                 1.into(),
@@ -609,9 +784,8 @@ mod test {
                         nil()
                     )
                 )
-            )
+            ), &Normal, 0
         );
-
         assert_eq!(list_pushed, list_consed);
     }
 
