@@ -132,7 +132,7 @@ impl Term {
     /// ```
     /// use lambda_calculus::term::Term::*;
     ///
-    /// assert_eq!(Var(0).app(Var(1)).unapp(), Ok((Var(0), Var(1))));
+    /// assert_eq!(Var(1).app(Var(2)).unapp(), Ok((Var(1), Var(2))));
     /// ```
     pub fn unapp(self) -> Result<(Term, Term), Error> {
         match self {
@@ -147,7 +147,7 @@ impl Term {
     /// ```
     /// use lambda_calculus::term::Term::*;
     ///
-    /// assert_eq!(Var(0).app(Var(1)).unapp_ref(), Ok((&Var(0), &Var(1))));
+    /// assert_eq!(Var(1).app(Var(2)).unapp_ref(), Ok((&Var(1), &Var(2))));
     /// ```
     pub fn unapp_ref(&self) -> Result<(&Term, &Term), Error> {
         match *self {
@@ -162,7 +162,7 @@ impl Term {
     /// ```
     /// use lambda_calculus::term::Term::*;
     ///
-    /// assert_eq!(Var(0).app(Var(1)).unapp_ref_mut(), Ok((&mut Var(0), &mut Var(1))));
+    /// assert_eq!(Var(1).app(Var(2)).unapp_ref_mut(), Ok((&mut Var(1), &mut Var(2))));
     /// ```
     pub fn unapp_ref_mut(&mut self) -> Result<(&mut Term, &mut Term), Error> {
         match *self {
@@ -177,7 +177,7 @@ impl Term {
     /// ```
     /// use lambda_calculus::term::Term::*;
     ///
-    /// assert_eq!(Var(0).app(Var(1)).lhs(), Ok(Var(0)));
+    /// assert_eq!(Var(1).app(Var(2)).lhs(), Ok(Var(1)));
     /// ```
     pub fn lhs(self) -> Result<Term, Error> {
         if let Ok((lhs, _)) = self.unapp() { Ok(lhs) } else { Err(NotAnApp) }
@@ -189,7 +189,7 @@ impl Term {
     /// ```
     /// use lambda_calculus::term::Term::*;
     ///
-    /// assert_eq!(Var(0).app(Var(1)).lhs_ref(), Ok(&Var(0)));
+    /// assert_eq!(Var(1).app(Var(2)).lhs_ref(), Ok(&Var(1)));
     /// ```
     pub fn lhs_ref(&self) -> Result<&Term, Error> {
         if let Ok((lhs, _)) = self.unapp_ref() { Ok(lhs) } else { Err(NotAnApp) }
@@ -201,7 +201,7 @@ impl Term {
     /// ```
     /// use lambda_calculus::term::Term::*;
     ///
-    /// assert_eq!(Var(0).app(Var(1)).lhs_ref_mut(), Ok(&mut Var(0)));
+    /// assert_eq!(Var(1).app(Var(2)).lhs_ref_mut(), Ok(&mut Var(1)));
     /// ```
     pub fn lhs_ref_mut(&mut self) -> Result<&mut Term, Error> {
         if let Ok((lhs, _)) = self.unapp_ref_mut() { Ok(lhs) } else { Err(NotAnApp) }
@@ -213,7 +213,7 @@ impl Term {
     /// ```
     /// use lambda_calculus::term::Term::*;
     ///
-    /// assert_eq!(Var(0).app(Var(1)).rhs(), Ok(Var(1)));
+    /// assert_eq!(Var(1).app(Var(2)).rhs(), Ok(Var(2)));
     /// ```
     pub fn rhs(self) -> Result<Term, Error> {
         if let Ok((_, rhs)) = self.unapp() { Ok(rhs) } else { Err(NotAnApp) }
@@ -225,7 +225,7 @@ impl Term {
     /// ```
     /// use lambda_calculus::term::Term::*;
     ///
-    /// assert_eq!(Var(0).app(Var(1)).rhs_ref(), Ok(&Var(1)));
+    /// assert_eq!(Var(1).app(Var(2)).rhs_ref(), Ok(&Var(2)));
     /// ```
     pub fn rhs_ref(&self) -> Result<&Term, Error> {
         if let Ok((_, rhs)) = self.unapp_ref() { Ok(rhs) } else { Err(NotAnApp) }
@@ -237,14 +237,14 @@ impl Term {
     /// ```
     /// use lambda_calculus::term::Term::*;
     ///
-    /// assert_eq!(Var(0).app(Var(1)).rhs_ref_mut(), Ok(&mut Var(1)));
+    /// assert_eq!(Var(1).app(Var(2)).rhs_ref_mut(), Ok(&mut Var(2)));
     /// ```
     pub fn rhs_ref_mut(&mut self) -> Result<&mut Term, Error> {
         if let Ok((_, rhs)) = self.unapp_ref_mut() { Ok(rhs) } else { Err(NotAnApp) }
     }
 }
 
-/// Wraps a term in an abstraction. Consumes its argument.
+/// Wraps a `Term` in an `Abs`traction. Consumes its argument.
 ///
 /// # Example
 /// ```
@@ -255,15 +255,15 @@ impl Term {
 /// ```
 pub fn abs(term: Term) -> Term { Abs(Box::new(term)) }
 
-/// Produces an application of its arguments without substitution or reduction, consuming them in
-/// the process.
+/// Produces an `App`lication of its `Term` arguments without any reduction, consuming them in the
+/// process.
 ///
 /// # Example
 /// ```
 /// use lambda_calculus::term::Term::*;
 /// use lambda_calculus::term::app;
 ///
-/// assert_eq!(app(Var(0), Var(1)), App(Box::new(Var(0)), Box::new(Var(1))));
+/// assert_eq!(app(Var(1), Var(2)), App(Box::new(Var(1)), Box::new(Var(2))));
 /// ```
 pub fn app(lhs: Term, rhs: Term) -> Term { App(Box::new(lhs), Box::new(rhs)) }
 
@@ -313,11 +313,39 @@ fn parenthesize_if(input: &str, condition: bool) -> Cow<str> {
     }
 }
 
+/// A macro for chain application of `Term`s.
+///
+/// # Example
+/// ```
+/// # #[macro_use] extern crate lambda_calculus;
+/// # fn main() {
+/// use lambda_calculus::term::Term::*;
+/// use lambda_calculus::term::app;
+///
+/// assert_eq!(app!(Var(1), Var(2), Var(3)), Var(1).app(Var(2)).app(Var(3)));
+/// # }
+/// ```
+#[macro_export]
+macro_rules! app {
+    ($term1:expr, $($term2:expr),+) => {
+        {
+            let mut term = $term1;
+            $(term = term.app($term2);)*
+            term
+        }
+    };
+}
+
 #[cfg(test)]
 mod test {
-    use super::DISPLAY_CLASSIC;
+    use super::{Var, DISPLAY_CLASSIC};
     use arithmetic::{zero, succ, pred};
 
+    #[test]
+    fn app_macro() {
+        assert_eq!(&format!("{}", app!(succ(), app!(Var(1), Var(2), Var(3)))), "(λλλ2(321))(123)");
+    }
+
     #[test]
     fn display_modes() {
         if DISPLAY_CLASSIC {
